<?php

defined( 'WPINC' ) or die;

if ( ! class_exists( 'Tribe__Snippet' ) ) {

	/**
	 * Abstract snippet class that handles common snippet tasks
	 */

	abstract class Tribe__Snippet {

		/**
		 * List of plugins required for this snippet to run
		 *
		 * Note: Snippet will not run unless all of these are activated
		 */
		protected $plugins_required = array();

		/**
		 * The base plugin file - usually set this to __FILE__
		 */
		protected $snippet_file;

		/**
		 * WordPress Action to initialize this snippet
		 *
		 * Common ones: plugins_loaded, init, wp_loaded, parse_request, parse_query, pre_get_posts
		 */
		protected $load_action = 'init';

		/**
		 * The text domain used by all i18n functions
		 * @TODO Add the ability to actually translate strings
		 */
		protected static $text_domain = 'tribe-snippet';

		/**
		 * Version number - Only set this if it must be available outside of /wp-admin/
		 *
		 * When in the admin area this will automatically be set to math the plugin file header via get_plugin_data()
		 */
		protected $plugin_version;

		/**
		 * Snippet name - Only set this if it must be available outside of /wp-admin/
		 *
		 * When in the admin area this will automatically be set via get_plugin_data()
		 */
		protected $plugin_name;

		/**
		 * Will automatically be set via get_plugin_data() on /wp-admin/ pages
		 */
		protected $plugin_info;

		/**
		 * A list of tribe plugin's details
		 *
		 * 'Plugin Short Name' => array(
		 *  'class'            => Main plugin class
		 *  'url'              => Download or purchase URL for plugin from within /wp-admin/ thickbox
		 *  'version_required' => Min version required by this snippet
		 *  'version_constant'  => Constant to retrieve plugin version from
		 *  )
		 */
		public $tribe_plugins = array(

			'Event Tickets' => array(
				'class' => 'Tribe__Tickets__Main',
				'url' => 'plugin-install.php?tab=plugin-information&plugin=event-tickets&TB_iframe=true',
				'version_required' => '4.0',
				'version_constant' => 'Tribe__Tickets__Main::VERSION',
			),

			'Event Tickets Plus' => array(
				'class' => 'Tribe__Tickets_Plus__Main',
				'url' => '//theeventscalendar.com/product/wordpress-event-tickets-plus/?TB_iframe=true',
				'version_required' => '4.0',
				'version_constant' => 'Tribe__Tickets_Plus__Main::VERSION',
			),

			'The Events Calendar' => array(
				'class' => 'Tribe__Events__Main',
				'url' => 'plugin-install.php?tab=plugin-information&plugin=the-events-calendar&TB_iframe=true',
				'version_required' => '4.0',
				'version_constant' => 'Tribe__Events__Main::VERSION',
			),

			'Events Calendar Pro' => array(
				'class' => 'Tribe__Events__Pro__Main',
				'url' => '//theeventscalendar.com/product/wordpress-events-calendar-pro/?TB_iframe=true',
				'version_required' => '4.0',
				'version_constant' => 'Tribe__Events__Pro__Main::VERSION',
			),

			'Community Events' => array(
				'class' => 'Tribe__Events__Community__Main',
				'url' => '//theeventscalendar.com/product/wordpress-community-events/?TB_iframe=true',
				'version_required' => '4.0',
				'version_constant' => 'Tribe__Events__Community__Main::VERSION',
			),

			'Community Tickets' => array(
				'class' => 'Tribe__Events__Community__Tickets__Main',
				'url' => '//theeventscalendar.com/product/community-tickets/?TB_iframe=true',
				'version_required' => '4.0',
				'version_constant' => 'Tribe__Events__Community__Tickets__Main::VERSION',
			),

			'Filter Bar' => array(
				'class' => 'Tribe__Events__Filterbar__View',
				'url' => '//theeventscalendar.com/product/wordpress-events-filterbar/?TB_iframe=true',
				'version_required' => '4.0',
				'version_constant' => 'Tribe__Events__Filterbar__View::VERSION',
			),

			'Facebook Events' => array(
				'class' => 'Tribe__Events__Facebook__Importer',
				'url' => '//theeventscalendar.com/product/facebook-events/?TB_iframe=true',
				'version_required' => '4.0',
				'version_constant' => 'Tribe__Events__Facebook__Importer::VERSION',
			),

			'iCal Importer' => array(
				'class' => 'Tribe__Events__Ical_Importer__Main',
				'url' => '//theeventscalendar.com/product/ical-importer/?TB_iframe=true',
			),

			'Eventbrite Tickets' => array(
				'class' => 'Tribe__Events__Tickets__Eventbrite__Main',
				'url' => '//theeventscalendar.com/product/wordpress-eventbrite-tickets/?TB_iframe=true',
			),

			'Advanced Post Manager' => array(
				'class' => 'Tribe_APM',
				'url' => 'plugin-install.php?tab=plugin-information&plugin=advanced-post-manager&TB_iframe=true',
			),

		);

		/**
		 * Tribe__Snippet constructor.
		 */
		public function __construct() {


			add_action( $this->load_action, array( $this, 'init' ) );
			add_action( 'admin_init', array( $this, 'admin_init' ) );
		}


		/*
		 * Initializer that will typically be an action hooked onto 'init'
		 */
		public function init() {

			// Bail if the required plugins are not present
			if ( $this->has_requisite_plugins() ) {
				$this->snippet_init();
			} else {
				add_action( 'admin_notices', array( $this, 'show_inactive_plugins_alert' ) );
			}
		}

		/*
		 * Sets up admin side of snippet
		 */
		public function admin_init(){
			// This makes the plugin data available when in the admin area
			$this->plugin_info = get_plugin_data( $this->snippet_file );
			if( ! isset( $this->plugin_name ) ) $this->plugin_name = $this->plugin_info['Name'];
			if( ! isset( $this->plugin_version ) ) $this->plugin_version = $this->plugin_info['Version'];
		}


		/**
		 * Abstract function for adding action and filter hooks
		 */
		abstract public function snippet_init();


		/**
		 * Shows an admin_notice message explaining why this snippet couldn't run
		 */
		public function show_inactive_plugins_alert() {

			if ( ! current_user_can( 'activate_plugins' ) ) {
				return;
			}

			$plugin_links = array();

			foreach ( $this->plugins_required as $key => $plugin ) {

				// Sometimes plugins_required is an array of plugin info rather than just a key
				$plugin = is_array( $plugin ) ? $key : $plugin;

				$plugin_links[] = sprintf(
					'<a href="%1$s" class="thickbox" title="%2$s">%3$s</a>',
					esc_attr( $this->tribe_plugins[ $plugin ][ 'url' ] ),
					esc_attr__( $plugin, self::$text_domain ),
					esc_html__( $plugin, self::$text_domain )
				);
			}

			printf(
				'<div class="error"><p>' . esc_html__( 'To begin using %1$s, please install and activate the latest version(s) of %2$s.', self::$text_domain ) . '</p></div>',
				$this->plugin_name,
				implode( ', ', $plugin_links )
			);

		}


		/**
		 * Checks if the required plugins (classes) exist
		 *
		 * @return boolean
		 */
		public function has_requisite_plugins() {

			foreach ( $this->plugins_required as $key => $req_plugin ) {

				// When a required plugin is an array override $this->tribe_plugins with new plugin data
				if( is_array( $req_plugin ) ) {
					$this->tribe_plugins = array_replace_recursive(
						$this->tribe_plugins,
						array( $key => $req_plugin )
					);

					// Now that the data is handled make $req_plugin a valid array key
					$req_plugin = $key;
				}

				$plugin_info = isset( $this->tribe_plugins[ $req_plugin ] ) ? $this->tribe_plugins[ $req_plugin ] : array();

				// Bail if main plugin class is not present
				if( ! isset( $plugin_info[ 'class' ] ) || ! class_exists( $plugin_info[ 'class' ] ) ) return false;

				// Bail if plugin's version is less than the required version
				if(
					isset( $plugin_info[ 'version_constant' ] ) &&
					isset( $plugin_info[ 'version_required' ] ) &&
					version_compare(
						constant( $plugin_info[ 'version_constant' ] ),
						$plugin_info[ 'version_required' ],
						'<'
					)
				) {
					return false;
				}

			} // end foreach

			return true;
		}

		/**
		 * Checks if the given plugin's main class is present
		 *
		 * @return boolean
		 */
		public function is_tribe_plugin_active( $plugin ) {
			return ( isset( $this->tribe_plugins[ $plugin ] ) && class_exists( $this->tribe_plugins[ $plugin ][ 'class' ] ) );
		}

	}
}